from typing import Union

MapdlInt = Union[
    str, int
]  # mapdl integers can either be strings or integers. "1" and 1 are equally valid
MapdlFloat = Union[
    str, float
]  # mapdl floats can either be strings or floats. "1.0" and 1.0 are equally valid
